require 'pg'
require 'yaml'

PRINT_QUERIES = ENV['PRINT_QUERIES'] == 'true'
MIGRATIONS = Dir.glob('./db/migrate/*.sql').to_a

  class DBConnection
    def self.app_name
      YAML.load_file(Dir.pwd + '/config/database.yml')['database']
    end

    def self.add_to_version(file)
      name = parse_migration_file(file)
      execute(<<-SQL, [name])
        INSERT INTO
          version (name)
        VALUES
          ($1);
      SQL
    end

    def self.columns(table_name)
      columns = instance.exec(<<-SQL)
        SELECT
          attname
        FROM
          pg_attribute
        WHERE
          attrelid = '#{table_name}'::regclass AND
          attnum > 0 AND
          NOT attisdropped
      SQL

      columns.map { |col| col['attname'].to_sym }
    end

    def self.ensure_version_table
        execute(<<-SQL)
          CREATE TABLE IF NOT EXISTS version (
            id SERIAL PRIMARY KEY,
            name VARCHAR(255) NOT NULL
          );
        SQL
    end

    def self.execute(query, params = [])
      query = number_placeholders(query)
      print_query(query, params)
      instance.exec(query, params)
    end

    def self.instance
      @db || open
    end

    def self.migrate
      ensure_version_table
      pending_migrations = MIGRATIONS.reject { |file| migrated?(file) }
      pending_migrations.each do |file|
        add_to_version(file)
        if (ENV['DATABASE_URL'])
          uri = URI.parse(ENV['DATABASE_URL'])
          db_name = uri.path[1..-1]
        else
          db_name = app_name
        end
        `psql -d #{db_name} -a -f #{file}`
      end
    end

    def self.migrated?(file)
      name = parse_migration_file(file)
      result = execute(<<-SQL, [name])
        SELECT
          *
        FROM
          version
        WHERE
          name = $1;
      SQL
      !!result.first
    end

    def self.parse_migration_file(file)
      filename = File.basename(file).split('.').first
      u_idx = filename.index('_')
      filename[0..u_idx - 1]
    end

    def self.number_placeholders(query_string)
      count = 0
      query_string.chars.map do |char|
        if char == "?"
          count += 1
          "$#{count}"
        else
          char
        end
      end.join("")
    end

    def self.print_query(query, interpolation_args)
      return unless PRINT_QUERIES

      puts '--------------------'
      puts query
      unless interpolation_args.empty?
        puts "interpolate: #{interpolation_args.inspect}"
      end
      puts '--------------------'
    end

    def self.open
      if (ENV['DATABASE_URL'])
        uri = URI.parse(ENV['DATABASE_URL'])

        @db = PG::Connection.new(
           user: uri.user,
           password: uri.password,
           host: uri.host,
           port: uri.port,
           dbname: uri.path[1..-1],
        )
      else
        @db = PG::Connection.new(
          dbname: app_name,
          port: 5432
        )
      end
    end

    def self.reset
      if (ENV['DATABASE_URL'])
        uri = URI.parse(ENV['DATABASE_URL'])

        commands = [
          "dropdb #{uri.path[1..-1]}",
          "createdb #{uri.path[1..-1]}"
        ]

      else
        commands = [
          "dropdb #{app_name}",
          "createdb #{app_name}"
        ]
      end

      commands.each { |command| `#{command}` }
    end
  end
